import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import sun.net.*;
import sun.net.ftp.*;

public class AsosAppCLI
{
   private String stationCode;
   private ZipOutputStream asosOut=null;

// '.jpg'=images, '.wav'=sound files, '.txt'=text files
   public static String INFO    = "INFO.txt";
   public static String WAVONE  = "1.wav";
   public static String WAVTWO  = "2.wav";
   public static String FIRST   = "000.jpg";
   public static String SECOND  = "045.jpg";
   public static String THIRD   = "090.jpg";
   public static String FOURTH  = "135.jpg";
   public static String FIFTH   = "180.jpg";
   public static String SIXTH   = "225.jpg";
   public static String SEVENTH = "270.jpg";
   public static String EIGHTH  = "315.jpg";
   public static String[] files = { WAVONE, WAVTWO, FIRST, 
                                    SECOND, THIRD, FOURTH, FIFTH,
                                    SIXTH, SEVENTH, EIGHTH, INFO };
   public static String[] directions = { "0/360", "045", "090", "135", "180", "225", "270", "315" };
   public static String[] questions  = { "Date of Documentaion: ", "WFO: ",
                                         "Name: ", "Phone: ", "Email Address: ",
                                         "ASOS Location: ", "ASOS Station ID: ",
                                         "ASOS Latitude (Deg.min.sec): ",
                                         "ASOS Longitude (Deg.min.sec): ",
                                         "ASOS Anemometer mast height (f): " };

   static FtpClient ftp;

   public static String EMAIL = "dbowman@ncdc.noaa.gov";
   public static String FTP   = "ftp.ncdc.noaa.gov";
   public static String LOGIN = "anonymous";
   public static String CD    = "pub/upload";
//   public static String CD    = "cd/pub/upload";

   private Vector fileList= new Vector();

   public AsosAppCLI( String statCode )
   {
      stationCode = statCode.trim().toUpperCase();
   } // end Constructor

   public String getCode()
   {
      return stationCode;
   } // end getCode

   public boolean testDirectory( String dir )
   {
      String filename=null;
      File tempFile=null;
      boolean missingFlag=false;
      boolean returnValue=true;
      int j=0;

      for(int i=0; i<files.length-1; i++)
      {
         if(i>2)
         {
            try {
               filename = stationCode + files[i];
               tempFile = new File( dir, filename );
               if(!tempFile.exists())
               {
                  System.out.println(" Missing file: " + filename);
                  missingFlag=true;
                  returnValue=false;
               }
               else
               {  fileList.addElement(filename);
                  j++; }
            } catch(Exception anException) { 
               System.out.println(" Missing file: " + filename);
               missingFlag = true;
               returnValue = false;
            }
         } else {
            try {
               filename = stationCode + files[i];
               tempFile = new File( dir, filename );
               if(tempFile.exists())
               {  fileList.addElement(filename);
                  j++; }
               else
               {  System.out.println(" Missing file: " + filename);
                  missingFlag=true; }
            } catch(Exception anException) { 
               System.out.println(" Missing file: " + filename);
               missingFlag=true;
            }
         }
      continue;
      }
      if(missingFlag)
      System.out.println("*****************************************************");

      return returnValue;
   } // end testDirectory()

   public boolean fileInit( String name, File f )
   {
      byte buff[] = new byte[1000];
      String ow = null;

      try {
         if(f.exists())
         {
            System.out.println("File " + name + " already Exist");
            System.out.print("Overwrite (y/n):");
            System.in.read( buff );
            ow = new String( buff ).trim().toLowerCase();
            if( ow.compareTo("n") == 0 ) {
               System.out.println("Please rename existing file and re-run program, Thank You.");
               return false;
            } else {
               System.out.println("Overwriting file " + name );
            }
         } else {
            System.out.println("File " + name + " does not exist");
            System.out.println("Creating File");
         }
      } catch(Exception anException) {
         System.out.println("Error occurred while initializing: " + name);
         return false;
      }
      return true;
   } // end fileAInit()

   public boolean createFile(String dir)
   {
      String filename       = new String( stationCode + INFO );
      byte buff[]           = new byte[1000];
      File info             = new File(dir, filename);
      FileOutputStream fout = null;
      PrintWriter pout      = null;

      if(fileInit( filename, info ))
      {
         System.out.println("*****************************************************");
         try {
            fout = new FileOutputStream( info );
            pout = new PrintWriter( fout );
            System.out.println("Please fill out the following form:");

            for (int i=0; i<questions.length;i++)
            {
               System.out.print(questions[i]);
               System.in.read( buff );
               pout.println(questions[i] + new String( buff ).trim() );             
               buff = new byte[1000];
            }
            System.out.println();
            pout.println();
            System.out.println("Compass     Description of Wind Exposure: (identification of Obstacles to the wind");
            System.out.println("Headding    flow, approx. distance from ASOS and estimated hieght of obstacles)");
            for(int i=0; i<directions.length;i++)
            {
               System.out.println(directions[i] + ":");
               System.in.read( buff );
               pout.println(directions[i] + ":");
               pout.println( new String( buff ).trim() );
               buff = new byte[1000];
               System.out.println();
               pout.println();
            }
            pout.flush();
            fout.close();
         } catch(Exception anException) {
            return false;
         }
      } else { return false; }
      fileList.addElement(filename);
      System.out.println("File: " + filename + " stored in directory " + dir);
      return true;
   } // end createFile()

   public boolean goZip( String dir )
   {
      String  zipName, ext = ".zip";
      FileOutputStream fos = null;
      FileInputStream fis  = null;
      byte tempBuff[]      = new byte[1000];

      zipName = new String( stationCode );
      zipName = zipName.trim();
      zipName = zipName.concat( ext );

      try {
         fos = new FileOutputStream( zipName );
         asosOut = new ZipOutputStream( fos ); 
         ZipEntry entry = new ZipEntry(stationCode + "/");
         asosOut.putNextEntry(entry);
         asosOut.closeEntry();
         System.out.println("Making compressed Archive " + zipName);
         for(int i=0; i<fileList.size(); i++)
         {
            System.out.println(" Deflating: " + (String)fileList.elementAt(i) + " to " +
                                 stationCode + "/" + (String)fileList.elementAt(i) );

            fis = new FileInputStream(dir + "/" + (String)fileList.elementAt(i) );
            entry = new ZipEntry(stationCode + "/" + (String)fileList.elementAt(i) );
            asosOut.putNextEntry(entry);

            int buffSize = 0;
            while( (buffSize=fis.read(tempBuff))>0){
               asosOut.write(tempBuff, 0, buffSize);
            }
            asosOut.flush();
            asosOut.closeEntry();
         }
         asosOut.flush();
         asosOut.close();

      } catch(Exception anException) {
         return false;
      }
      return true;

   } // end goZip()

   public boolean ftpConnect()
   {
      String file      = stationCode+".zip";
      String email     = null;
      String ftpSite   = null;
      byte buff[]      = new byte[100];
      StringBuffer buf = new StringBuffer();
      int ch;
      FileInputStream ftpFile = null;
      byte tempBuff[] = new byte[1000];
      FtpClient ftp=null;
      BufferedOutputStream ftpOut=null;

      try {
         System.out.print("Please input FTP site [" + FTP + "]: ");
            System.in.read( buff );
            ftpSite = new String( buff ).trim();
            buff = new byte[100];

         System.out.print("Please input your email address: ");
            System.in.read( buff );
            email = new String( buff );
            email = email.trim();

         if(ftpSite.equals( new String(""))) {
            ftp = new FtpClient( FTP );
            System.out.println("Using: " + FTP);
         } else {
            ftp = new FtpClient( ftpSite );
            System.out.println("Using: " + ftpSite);
         }

         ftp.login( LOGIN, email );
         ftp.cd( CD );

         ftp.binary();
         System.out.println(" Uploading File " + file);

         int buffSize = 0;
         ftpFile = new FileInputStream(new File(file));
         ftpOut = ftp.put(file);
         while( (buffSize=ftpFile.read(tempBuff)) >0){ 
            ftpOut.write(tempBuff, 0, buffSize);
         }

         ftpOut.close();
         ftp.closeServer();

      } catch(Exception anException) {
         System.err.println("FTP: " + anException.getMessage());
         return false;
      }
      return true;
   } // end ftpConnect()

   public boolean go()
   {
      byte buff[] = new byte[100];
      String directory;
      String missingFiles[] = new String[files.length];

      System.out.print("Please input name of path(directory) containing asos files: ");
      try {
         System.in.read( buff );
      } catch(Exception anException) {
         System.err.println("Dir: " + anException.getMessage());
         anException.printStackTrace();
      }
      directory = new String( buff );
      directory = directory.trim();

      System.out.println("*****************************************************");
      if(testDirectory( directory )) {
         if(createFile( directory )) {
            System.out.println("*****************************************************");
            if(goZip( directory )) {
               System.out.println("*****************************************************");
               return true; }}}

      return false;
   } // end go

   public static void main( String argv[] )
   {
      byte buff[] = new byte[100];
      AsosApp asos = null;
      String stat;

      try {
         System.out.println("ASOS Page Formatting Application");
         System.out.print("Please input station 3-letter code (ie: MIA ): ");
         System.in.read( buff );
     
         stat = new String( buff );
         asos = new AsosApp( stat );

         if(asos.go())
         {
            if(asos.ftpConnect())
            {
               System.out.println(" File uploaded ");
               System.out.println("*****************************************************"); 
               System.out.println("Job Done");
               System.out.println("Please now send Email to Dave Bowman at: ");
               System.out.println("\t" + EMAIL);
               System.out.print("Please include in your message the filename: ");
               System.out.println( asos.getCode() +".zip");
            } else {
               System.out.println(" Transfer not completed ");
               System.out.println("*****************************************************");
               System.out.println("Job Not Done");
            }
         } else {
            System.out.println("*****************************************************");
            System.out.println("Job Not Done");
         }

      } catch(Exception anException) {
         System.err.println("main: " + anException.getMessage());
         System.exit(0);
      }
   } // end main
}
