//
//  AsosAppController.java
//  AsosApp
//
//  Created by Nick Carrasco on Thu May 08 2003.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.*;

public class AsosAppController extends JPanel implements ActionListener {
  private AsosImageLoaderPanel imageLoaderPanel;
  private AsosSoundLoaderPanel soundLoaderPanel;
  private AsosStationInfoPanel stationInfoPanel;
  private AsosExposurePanel exposurePanel; 
  
  private JFileChooser chooser;
  private String wmoID;
  private File[] files; // and arraylist for use with creating the zip file
  private File textFile; // temporary text file for storing/writing ascii info to
  private boolean infoSet; // set flag for knowing when the info stuff has been set
  private boolean expSet; // set flag for knowing when the exposure stuff has been set
  private String asciiInfoData; // one really long string for hold info data
  private String asciiExpData; // one really long string for hold exposure data
  private File zipArchive; // the actual archive to be sent
  private String emailMessage; // message to be sent to asos photo site manager

//  private String remoteHost = new String("ftp.ncdc.noaa.gov");
  private String remoteHost = new String("ftp.aoml.noaa.gov");
//  private String remoteDir = new String("pub/upload");
  private String remoteDir = new String("/hrd/incoming/carrasco");
//  private String managerEmail = new String("David.P.Bowman@noaa.gov");
  private String managerEmail = new String("Nick.Carrasco@noaa.gov");
  
  private JLabel statusLabel;
  private JButton sendButton;
  
  public AsosAppController() {
    files = new File[11]; // 0->ascii; [1|2]->[1|2].wav; [3-10]->[360|045|...|315].jpg
    chooser = new JFileChooser();  
  
    this.setLayout(new BorderLayout());
    
    JTabbedPane asosPanes = new JTabbedPane();

    stationInfoPanel = new AsosStationInfoPanel(this);
    asosPanes.add("Station Information",stationInfoPanel);

    exposurePanel = new AsosExposurePanel(this);
    asosPanes.add("Exposure Comments",exposurePanel);

    Box loaderBox = Box.createVerticalBox();
      imageLoaderPanel = new AsosImageLoaderPanel(this);
      loaderBox.add(imageLoaderPanel);    
      soundLoaderPanel = new AsosSoundLoaderPanel(this);
      loaderBox.add(soundLoaderPanel);
      JPanel loadDirPanel = new JPanel();
        loadDirPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        JButton loadDirButton = new JButton("Load Directory");
        loadDirButton.setActionCommand("loaddir");
        loadDirButton.addActionListener(this);
        loadDirPanel.add(loadDirButton);
        loadDirPanel.setVisible(true);
      loaderBox.add(loadDirPanel);
    asosPanes.add("Load Station Files",loaderBox);
    
    this.add(asosPanes,BorderLayout.CENTER);
    
    Box controllerBox = Box.createHorizontalBox();
      statusLabel = new JLabel("Status: ...");
      controllerBox.add(statusLabel);
      controllerBox.add(Box.createHorizontalGlue());
      sendButton = new JButton("Compress and Send");
      sendButton.setActionCommand("send");
      sendButton.addActionListener(this);
      controllerBox.add(sendButton);
    this.add(controllerBox,BorderLayout.SOUTH);
   
    this.setVisible(true);
  }


  public void actionPerformed(ActionEvent e) {
    Object eventSource = e.getSource();
    String eventString = e.getActionCommand();
    statusLabel.setText(eventString);

    // following is an ugly if else statement for determining which event is which
    if(eventString.equalsIgnoreCase("wmoid")) {
      wmoID=new String(stationInfoPanel.getStationID());
statusLabel.setText(stationInfoPanel.getStationID());
      imageLoaderPanel.setStationLabelText(stationInfoPanel.getStationID());
    }
    
    // retrieve information for use with creating ascii file
    else if(eventString.equalsIgnoreCase("statinfo")) {
statusLabel.setText("getting station info");
      asciiInfoData = stationInfoPanel.getAsciiInfo();
      if(asciiInfoData==null) {
        JOptionPane.showMessageDialog(this,
            "<HTML><FONT color=\"RED\">Please check that all fields in the Station Info Pane are filled</FONT></HTML>",
            "Error",JOptionPane.WARNING_MESSAGE);
      } else infoSet=true;        
    } 

    else if(eventString.equalsIgnoreCase("expinfo")) {
statusLabel.setText("getting exposure info");
      asciiExpData = exposurePanel.getAsciiInfo();
      if(asciiExpData==null) {
         JOptionPane.showMessageDialog(this,
             "<HTML><FONT color=\"RED\">Please fill in the exposure comments</FONT></HTML>",
             "Warning",JOptionPane.WARNING_MESSAGE);
      } else expSet=true;        
    } 

    // actions to be performed when loading and entire directory of files
    else if(eventString.equalsIgnoreCase("loaddir")) {
      chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        File directory = chooser.getSelectedFile();
        // get files from selected directory and update buttons.
        getFilesFromDirectory(directory);
      }
      chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    } 
    
    // actions to be performed when loading files
    else if(eventString.equalsIgnoreCase("360")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[3] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage360Icon(files[3].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("045")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[4] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage045Icon(files[4].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("090")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[5] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage090Icon(files[5].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("135")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[6] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage135Icon(files[6].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("180")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[7] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage180Icon(files[7].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("225")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[8] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage225Icon(files[8].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("270")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[9] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage270Icon(files[9].getAbsolutePath());
      }
    }
    
    else if(eventString.equalsIgnoreCase("315")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[10] = chooser.getSelectedFile();
        imageLoaderPanel.setLoadImage315Icon(files[10].getAbsolutePath());
      }
    } 
    
    else if(eventString.equalsIgnoreCase("location")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[1] = chooser.getSelectedFile();
      }
    } 
    
    else if(eventString.equalsIgnoreCase("obstacle")) {
      int returnVal = chooser.showOpenDialog(this);
      if(returnVal == JFileChooser.APPROVE_OPTION) {
statusLabel.setText(chooser.getSelectedFile().getName());
        files[2] = chooser.getSelectedFile();
      }
    }
      
    else if(eventString.equalsIgnoreCase("send")) {
statusLabel.setText("creating ascii file");
      if(createAsciiFile()==false) { 
        JOptionPane.showMessageDialog(this,
            "<HTML><FONT color=\"RED\">Please check that all fields in the Station and Exposure Panes are filled</FONT></HTML>",
            "Error",JOptionPane.WARNING_MESSAGE);
        return;
      }
statusLabel.setText("creating zip file");
      if(createZipArchive()==false) {
        JOptionPane.showMessageDialog(this,
            "<HTML><FONT color=\"RED\">Please check that you selected all the various files</FONT></HTML>",
            "Error",JOptionPane.WARNING_MESSAGE);
        return;
      }
statusLabel.setText("ftping zip file");
      if(sendZipArchive()==false) {
        JOptionPane.showMessageDialog(this,
            "<HTML><FONT color=\"RED\">Error sending zipped station infor, please try again later</FONT></HTML>",
            "Error",JOptionPane.WARNING_MESSAGE);
        return;
      } 
statusLabel.setText("generating email to " + managerEmail);
      emailMessage = stationInfoPanel.getEmailInfo();      
      
    }
  }
  
  void getFilesFromDirectory(File directory){ 
    if(directory.isDirectory()) {
      File[] dirFiles = directory.listFiles();
      for(int i=0;i<dirFiles.length;i++) {
        String fileName = dirFiles[i].getName();
        if(fileName.indexOf("000")!=-1 || fileName.indexOf("360")!=-1) { 
          files[3]=dirFiles[i];
          imageLoaderPanel.setLoadImage360Icon(files[3].getAbsolutePath());
        } else if(fileName.indexOf("045")!=-1) {
          files[4]=dirFiles[i];
          imageLoaderPanel.setLoadImage045Icon(files[4].getAbsolutePath());
        } else if(fileName.indexOf("090")!=-1) {
          files[5]=dirFiles[i];
          imageLoaderPanel.setLoadImage090Icon(files[5].getAbsolutePath());
        } else if(fileName.indexOf("135")!=-1) {
          files[6]=dirFiles[i];
          imageLoaderPanel.setLoadImage135Icon(files[6].getAbsolutePath());
        } else if(fileName.indexOf("180")!=-1) {
          files[7]=dirFiles[i];
          imageLoaderPanel.setLoadImage180Icon(files[7].getAbsolutePath());
        } else if(fileName.indexOf("225")!=-1) {
          files[8]=dirFiles[i];
          imageLoaderPanel.setLoadImage225Icon(files[8].getAbsolutePath());
        } else if(fileName.indexOf("270")!=-1) {
          files[9]=dirFiles[i];
          imageLoaderPanel.setLoadImage270Icon(files[9].getAbsolutePath());
        } else if(fileName.indexOf("315")!=-1) {
          files[10]=dirFiles[i];
          imageLoaderPanel.setLoadImage315Icon(files[10].getAbsolutePath());
        } else if(fileName.indexOf("1.wav")!=-1) {
          files[1]=dirFiles[i];
        } else if(fileName.indexOf("2.wav")!=-1) {
          files[2]=dirFiles[i];
        }
      }
    }
  }
  
  boolean createAsciiFile() {
    try {
      textFile = new File(wmoID+"info.txt");
      textFile.deleteOnExit();
//      textFile = File.createTempFile(wmoID+"info",null);
//      textFile = new File(wmoID+"info.txt");
      BufferedWriter writer = new BufferedWriter(new FileWriter(textFile));
      if(infoSet==false) {
        JOptionPane.showMessageDialog(this,
            "<HTML><FONT color=\"RED\">Please check that all fields in the Station Info Pane are filled</FONT></HTML>",
            "Error",JOptionPane.WARNING_MESSAGE);
        return false;
      } else writer.write(stationInfoPanel.getAsciiInfo());
      writer.flush();
      if(expSet==false) {
         JOptionPane.showMessageDialog(this,
             "<HTML><FONT color=\"RED\">Please fill in the exposure comments</FONT></HTML>",
             "Warning",JOptionPane.WARNING_MESSAGE);
          return false ;
      } else writer.write(exposurePanel.getAsciiInfo());
      writer.close();
      files[0] = textFile;
    } catch(Exception e) {
      e.printStackTrace();
    }
    return true;
  }

  // routine to create a zip archive from an array of files (File[])
  boolean createZipArchive() {
    for(int i=0;i<files.length;i++) {
      if(files[i]==null) return false;
    }
    
    try {
      // create zip archive file
      zipArchive = File.createTempFile(wmoID,".zip");
      
      // create stream for writing to zip archive
      ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(zipArchive));
      
      // create "root" directory for zip archive ie: stationname/
      zos.putNextEntry(new ZipEntry(wmoID + "/")); 
      
      // loop through file array and add each file to archive while displaying progress in status label
      for(int i=0;i<files.length;i++) {
statusLabel.setText(i + " Adding " + files[i].getName() + " to " + wmoID + "/" + files[i].getName());
        FileInputStream fis = new FileInputStream(files[i]);
        ZipEntry entry;
        if(i<3) entry = new ZipEntry(wmoID + "/" + files[i].getName());
        else entry = new ZipEntry(wmoID + "/" + wmoID + files[i].getName());

        zos.putNextEntry(entry);

        // copy file to zip archive through use of buffers
        int bufferSize = 0;
        byte[] buffer = new byte[1000]; 
        while((bufferSize = fis.read(buffer))>0) {
          zos.write(buffer,0,bufferSize);
        }
        zos.flush();
        zos.closeEntry();
      }
      zos.flush();
      zos.close();
    } catch(Exception e) {
      e.printStackTrace();
    }
    return true;
  }
  
  // routine used to ftp file to the desired server (hardcoded)
  boolean sendZipArchive() {
    String user = new String("anonymous");
    String passwd = stationInfoPanel.getEmail();
    String fileName = new String(wmoID+".zip");
    
    FTPSession ftpClient;
    try {
      ftpClient = new FTPSession(remoteHost,user,passwd);
      ftpClient.logon();
      
      ftpClient.put(fileName,remoteDir,zipArchive);
      
      ftpClient.close();
    } catch(Exception e) {
      e.printStackTrace();
    }
    return true;
  }
}

