//
//  FTPSession.java
//  AsosApp
//
//  Not Created by Nick Carrasco on Mon May 19 2003.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
// 
// this source code basically comes from a port by "manhthang"  at experts-exchange.com
//

import java.io.*;
import java.net.*;
import java.util.*;

public class FTPSession {
  public String host;   // Host name we connect to
  public int port;   // port number we connect to, default=21
  public String username;   // Username to log in with
  public String password;   // Password to log in with
  public String acct;   // optional acct for logging in

  protected Socket sessionSock;   // The control socket

//  protected DataInputStream inStream;
  protected BufferedReader inStream;
//  protected DataOutputStream outStream;
  protected DataOutputStream outStream;

  public FTPSession()
  {
  }

// This should be the most common constructor, it opens up an FTP session
// with the named host using the default FTP port and logs on using
// the username and password.

  public FTPSession(String host, String username, String password)
  throws IOException
  {
     this.host = host;
     this.port = 21;   // default FTP port is 21

     this.username = username;
     this.password = password;

     logon();   // go ahead and log on to the server
  }

// Opens up an FTP session with the named host using the default FTP
// port and logs on using the username and password, and account.
// The account field is pretty rare on FTP servers.

  public FTPSession(String host, String username, String password,
      String acct)
  throws IOException
  {
     this.host = host;
     this.port = 21;   // default FTP port is 21

     this.username = username;
     this.password = password;
     this.acct = acct;

     logon();   // go ahead and log on to the server
  }

// Opens up an FTP session with the named host using an alternate
// port number and if logOn is true, logs on using the username and password.

  public FTPSession(String host, int port, String username,
     String password, boolean logOn)
  throws IOException
  {
     this.host = host;
     this.port = port;
     if (this.port <= 0) this.port = 21;

     this.username = username;
     this.password = password;

     if (logOn) {
        logon();
     }
  }

// Opens up an FTP session with the named host using an alternate
// port number and if logOn is true, logs on using the username,
// password and account.

  public FTPSession(String host, int port, String username,
     String password, String acct, boolean logOn)
  throws IOException
  {
     this.host = host;
     this.port = port;
     if (this.port <= 0) this.port = 21;

     this.username = username;
     this.password = password;
     this.acct = acct;

     if (logOn) {
        logon();
     }
  }

// Close down the session

  public void close()
  throws IOException
  {
     sessionSock.close();
     sessionSock = null;
  }

// Connect to the server

  public void connect()
  throws IOException
  {
     sessionSock = new Socket(host, port);
//     inStream = new DataIntputStream(sessionSock.getInputStream());
     inStream = new BufferedReader(new InputStreamReader(sessionSock.getInputStream()));
     outStream = new DataOutputStream(sessionSock.getOutputStream());

  }

// Send a command and wait for a response

  public String doCommand(String commandString)
  throws IOException
  {
     outStream.writeBytes(commandString+"\n");
     String response = getResponse();
     return response;
  }

// Get a response back from the server. Handles multi-line responses
// and returns them as part of the string.

  public String getResponse()
  throws IOException
  {
     String response = "";

     for (;;) {
        String line = inStream.readLine();

        if (line == null) {
           throw new IOException(
              "Bad response from server.");
        }

// FTP response lines should at the very least have a 3-digit number

        if (line.length() < 3) {
           throw new IOException(
              "Bad response from server.");
        }
        response += line + "\n";

// If there isn't a '-' immediately after the number, we've gotten the
// complete response. ('-' is the continuation character for FTP responses)

        if ((line.length() == 3) ||
           (line.charAt(3) != '-')) return response;
     }
  }


// Logs on to the FTP server

  public void logon()
  throws IOException
  {
     connect();

// After connecting, the FTP server will send a response string. Make
// sure it starts with a '2' (reponses in the 200's are positive
// responses.

     String response = getResponse();
     if (response.charAt(0) != '2') {
        throw new IOException(response);
     }

// Send a logon command
     response = doCommand("USER "+username);

// If we get a response in the 300's, send a password

     if (response.charAt(0) == '3') {
        response = doCommand("PASS "+password);

// If we get a response in the 300's on the password command,
// send the account
        if (response.charAt(0) == '3') {
           response = doCommand("ACCT "+acct);
        }
     }

// If the last response we got wasn't in the 200's, there was an error
// during the logon.

     if (response.charAt(0) != '2') {
        throw new IOException(response);
     }
  }

// Creates a data connection to the server by using the PASV command.
// Normally the data connection is set up by the client sending the
// server an address and port number using the PORT command. Unfortunately,
// since an applet cannot listen for incoming connections, we can't work
// that way. The PASV command asks the server to accept a connection.
// The response for the PASV command contains a host address and port
// number in the form h,h,h,h,p,p where h is a byte in the host address
// and p is a byte in the port.

  protected synchronized Socket doPasvPort()
  throws IOException
  {

// Send the PASV command
     String response = doCommand("PASV");

// If it wasn't in the 200's, there was an error

     if (response.charAt(0) != '2') {
        throw new IOException(response);
     }

// The pasv response looks like:
// 227 Entering Passive Mode (127,0,0,1,4,160)
// We'll look for the ()'s at the end first

     int parenStart = response.lastIndexOf('(');
     int parenEnd = response.lastIndexOf(')');

// Make sure they're both there and that the ) comes after the (
     if ((parenStart < 0) || (parenEnd < 0) ||
        (parenStart >= parenEnd)) {
        throw new IOException("PASV response format error");
     }

// Extract the address bytes
     String pasvAddr = response.substring(parenStart+1, parenEnd);

// Create a tokenizer to parse the bytes
     StringTokenizer tokenizer = new StringTokenizer(pasvAddr, ",");

// Create the array to store the bytes
     int[] addrValues = new int[6];

// Parse each byte
     for (int i=0; (i < 6) && tokenizer.hasMoreTokens(); i++) {
        try {
           addrValues[i] = Integer.valueOf(
              tokenizer.nextToken()).intValue();
        } catch (Exception e) {
           throw new IOException(
              "PASV response format error");
        }
     }

// We ignore the host addresses, assuming that the host address is
// the same as the host address we used to connect the first time.

     Socket newSock = new Socket(host, (addrValues[4] << 8) +
        addrValues[5]);

     return newSock;
  }

// Fetches a file in binary mode and returns it as an array of bytes

  public byte[] get(String remoteFile)
  throws IOException
  {
     return get(remoteFile, true);   // default to binary
  }

// Fetches a file in binary mode if doBinary is true, or ascii mode if
// it's false, and returns it as an array of bytes.

  public synchronized byte[] get(String remoteFile, boolean doBinary)
  throws IOException
  {

// If transferring in binary mode, send a type command for type I (IMAGE)
     if (doBinary) {
        String response = doCommand("TYPE I");
        if (response.charAt(0) != '2') {
           throw new IOException(response);
        }
// If transferring in ascii mode, send a type command for type A (ASCII)
     } else {
        String response = doCommand("TYPE A");
        if (response.charAt(0) != '2') {
           throw new IOException(response);
        }
     }

// Set up the data connection

     Socket getSock = doPasvPort();

// Tell the server to send the file over
     String response = doCommand("RETR "+remoteFile);

// If the request is successful, the server should send a response
// in the 100's and then start sending the file. Once the file is
// sent, it should send a response in the 200's.

// Check for an initial response in the 100's

     if (response.charAt(0) != '1') {
        getSock.close();
        throw new IOException(response);
     }

     ByteArrayOutputStream out = new ByteArrayOutputStream();

// For binary transfers, read one byte at a time and store it in
// the array.
     if (doBinary) {
        InputStream in = getSock.getInputStream();

        int ch;

        while ((ch = in.read()) >= 0) {
           out.write(ch);
        }

// For ascii transfers, read a line at a time and strip off whatever
// newline we got.
     } else {
//        DataInputStream in = new DataInputStream(getSock.getInputStream());
        BufferedReader in = new BufferedReader(new InputStreamReader(getSock.getInputStream()));
        PrintStream dataOut = new PrintStream(out);

        String line;

        while ((line = in.readLine()) != null) {
           dataOut.println(line);
        }
     }

// Close the data connection
     getSock.close();

// Make sure we got a response in the 200's saying the transfer was
// successful.
     response = getResponse();
     if (response.charAt(0) != '2') {
        throw new IOException(response);
     }

     return out.toByteArray();
  }

// Stores an array of bytes in the named file using binary mode

  public synchronized void put(String remoteFile, byte[] data)
  throws IOException
  {
     put(remoteFile, data, true);   // default to binary
  }

// Stores an array of bytes in the named file, using binary mode if
// doBinary is true, ascii mode otherwise.

  public synchronized void put(String remoteFile, byte[] data,
     boolean doBinary)
  throws IOException
  {

// If transferring in binary mode, send a type command for type I (IMAGE)
     if (doBinary) {
        String response = doCommand("TYPE I");
        if (response.charAt(0) != '2') {
           throw new IOException(response);
        }

// If transferring in ascii mode, send a type command for type A (ASCII)
     } else {
        String response = doCommand("TYPE A");
        if (response.charAt(0) != '2') {
           throw new IOException(response);
        }
     }

// Open up a data connection
     Socket putSock = doPasvPort();

// Tell the server where we want it to store the data we are sending

     String response = doCommand("STOR "+remoteFile);

// If the request is successful, the server should send a response
// in the 100's and then start receiving the bytes. Once the data
// connection is closed, it should send a response in the 200's.

     if (response.charAt(0) != '1') {
        putSock.close();
        throw new IOException(response);
     }

// If binary mode, just write all the bytes
     if (doBinary) {
        OutputStream out = putSock.getOutputStream();

        out.write(data);

// If ascii mode, write the data a line at a time

     } else {
//        DataInputStream in = new DataInputStream(new ByteArrayInputStream(data));
        BufferedReader in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(data)));
        DataOutputStream out = new DataOutputStream(putSock.getOutputStream());

        String line;

        while ((line = in.readLine()) != null) {
           out.writeBytes(line+"\r");
        }
     }

     putSock.close();

     response = getResponse();

// Make sure we got a 200 response

     if (response.charAt(0) != '2') {
        throw new IOException(response);
     }
  }

// Puts a file in the named file on the remote server, only binary

  public synchronized void put(String remoteFile,String remoteDir, File file)
  throws IOException
  {

// transfer only in binary mode, send a type command for type I (IMAGE)
    String response = doCommand("TYPE I");
    if (response.charAt(0) != '2') {
      throw new IOException(response);
    }

// Open up a data connection
     Socket putSock = doPasvPort();

// Tell the server where we want it to store the data we are sending
     response = doCommand("CWD " +remoteDir);

// If the request is successful, the server should send a response
// in the 100's and then start receiving the bytes. Once the data
// connection is closed, it should send a response in the 200's.

    if (response.charAt(0) != '2') {
        putSock.close();
        throw new IOException(response);
     }

     response = doCommand("STOR "+remoteFile);

// If the request is successful, the server should send a response
// in the 100's and then start receiving the bytes. Once the data
// connection is closed, it should send a response in the 200's.

     if (response.charAt(0) != '1') {
        putSock.close();
        throw new IOException(response);
     }

// write file to server/host
    OutputStream out = putSock.getOutputStream();
    FileInputStream fis = new FileInputStream(file);
    int bufferSize = 0;
    byte[] buffer = new byte[1000]; 
    while((bufferSize = fis.read(buffer))>0) {
      out.write(buffer,0,bufferSize);
    }


    putSock.close();

    response = getResponse();

// Make sure we got a 200 response

    if (response.charAt(0) != '2') {
    throw new IOException(response);
    }
  }

}
